@@ -1,4 +1,5 @@
 """Internally used rule classes."""
+
 from __future__ import annotations
 
 import inspect
@@ -6,9 +7,12 @@ import logging
 from pathlib import Path
 from typing import TYPE_CHECKING, Any
 
+from packaging.version import InvalidVersion, Version
+
 from ansiblelint.constants import RULE_DOC_URL
 
 if TYPE_CHECKING:
+    from ansiblelint.config import Options
     from ansiblelint.errors import MatchError
     from ansiblelint.file_utils import Lintable
     from ansiblelint.rules import RulesCollection
@@ -39,11 +43,13 @@ class BaseRule:
     id: str = ""
     tags: list[str] = []
     description: str = ""
-    version_added: str = ""
+    version_changed: str = ""
     severity: str = ""
     link: str = ""
     has_dynamic_tags: bool = False
     needs_raw_task: bool = False
+    # Used to mark rules that we will never unload (internal ones)
+    unloadable: bool = False
     # We use _order to sort rules and to ensure that some run before others,
     # _order 0 for internal rules
     # _order 1 for rules that check that data can be loaded
@@ -52,9 +58,18 @@ class BaseRule:
     _help: str | None = None
     # Added when a rule is registered into a collection, gives access to options
     _collection: RulesCollection | None = None
+    # Allow rules to provide a custom short description instead of using __doc__
+    _shortdesc: str = ""
+
+    def __init__(self) -> None:
+        try:
+            Version(self.version_changed)
+        except InvalidVersion:
+            msg = f"Rule {self.__class__.__name__} has an invalid version_changed field '{self.version_changed}', is should be a 'X.Y.Z' format value."
+            _logger.warning(msg)
 
     @property
-    def help(self) -> str:  # noqa: A003
+    def help(self) -> str:
         """Return a help markdown string for the rule."""
         if self._help is None:
             self._help = ""
@@ -79,7 +94,7 @@ class BaseRule:
     @property
     def shortdesc(self) -> str:
         """Return the short description of the rule, basically the docstring."""
-        return self.__doc__ or ""
+        return self._shortdesc or self.__doc__ or ""
 
     def getmatches(self, file: Lintable) -> list[MatchError]:
         """Return all matches while ignoring exceptions."""
@@ -92,10 +107,11 @@ class BaseRule:
                     _logger.warning(
                         "Ignored exception from %s.%s while processing %s: %s",
                         self.__class__.__name__,
-                        method,
+                        method.__name__,
                         str(file),
                         exc,
                     )
+                    _logger.debug("Ignored exception details", exc_info=True)
         else:
             matches.extend(self.matchdir(file))
         return matches
@@ -143,7 +159,7 @@ class BaseRule:
 
     def __lt__(self, other: BaseRule) -> bool:
         """Enable us to sort rules by their id."""
-        return (self._order, self.id) < (other._order, other.id)  # noqa: SLF001
+        return (self._order, self.id) < (other._order, other.id)
 
     def __repr__(self) -> str:
         """Return a AnsibleLintRule instance representation."""
@@ -157,6 +173,26 @@ class BaseRule:
         """
         return getattr(cls, "_ids", {cls.id: cls.shortdesc})
 
+    @property
+    def rule_config(self) -> dict[str, Any]:
+        """Retrieve rule specific configuration."""
+        rule_config = {}
+        if self.options:
+            rule_config = self.options.rules.get(self.id, {})
+        if not isinstance(rule_config, dict):  # pragma: no branch
+            msg = f"Invalid rule config for {self.id}: {rule_config}"
+            raise RuntimeError(msg)  # noqa: TRY004
+        return rule_config
+
+    @property
+    def options(self) -> Options | None:
+        """Used to access linter configuration."""
+        if self._collection is None:
+            msg = f"A rule ({self.id}) that is not part of a collection cannot access its configuration."
+            _logger.warning(msg)
+            return None
+        return self._collection.options
+
 
 # pylint: enable=unused-argument
 
@@ -165,11 +201,12 @@ class RuntimeErrorRule(BaseRule):
     """Unexpected internal error."""
 
     id = "internal-error"
-    shortdesc = "Unexpected internal error"
+    _shortdesc = "Unexpected internal error"
     severity = "VERY_HIGH"
     tags = ["core"]
-    version_added = "v5.0.0"
+    version_changed = "5.0.0"
     _order = 0
+    unloadable = True
 
 
 class AnsibleParserErrorRule(BaseRule):
@@ -179,8 +216,9 @@ class AnsibleParserErrorRule(BaseRule):
     description = "Ansible parser fails; this usually indicates an invalid file."
     severity = "VERY_HIGH"
     tags = ["core"]
-    version_added = "v5.0.0"
+    version_changed = "5.0.0"
     _order = 0
+    unloadable = True
 
 
 class LoadingFailureRule(BaseRule):
@@ -190,12 +228,13 @@ class LoadingFailureRule(BaseRule):
     description = "Linter failed to process a file, possible invalid file."
     severity = "VERY_HIGH"
     tags = ["core", "unskippable"]
-    version_added = "v4.3.0"
+    version_changed = "4.3.0"
     _help = LOAD_FAILURE_MD
     _order = 0
     _ids = {
         "load-failure[not-found]": "File not found",
     }
+    unloadable = True
 
 
 class WarningRule(BaseRule):
@@ -205,5 +244,6 @@ class WarningRule(BaseRule):
     severity = "LOW"
     # should remain experimental as that would keep it warning only
     tags = ["core", "experimental"]
-    version_added = "v6.8.0"
+    version_changed = "6.8.0"
     _order = 0
+    unloadable = True
